#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.move import Move
from pycatia.system_interfaces.system_service import SystemService


class Position(Move):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     InfInterfaces.Move
                |                         Position
                | 
                | Represents the position object.
                | The position object is the 3D-axis system associated with an
                | object.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.position = com_object

    def get_components(self) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetComponents(CATSafeArrayVariant
                | oAxisComponentsArray)
                | 
                |     Returns the components of an object's position. This returns the 3D-axis
                |     system associated with the object.
                | 
                |     Parameters:
                | 
                |         oAxisComponentsArray
                |             The array used to store the twelve components retrieved from the
                |             objet's position. The first nine represent succcessively the components of the
                |             x-axis, y-axis, and z-axis. The last three represent the coordinates of the
                |             origin point. 
                | 
                |     Example:
                | 
                |           This example retrieves in oAxisComponentsArray
                |          the 3D-axis system components from 
                |          the Position object associated with MyObject:
                |          
                | 
                |          Dim oAxisComponentsArray ( 11 )
                |          MyObject.Position.GetComponents oAxisComponentsArray

        :return: tuple
        :rtype: tuple
        """
        vba_function_name = 'get_components'
        vba_code = """
        Public Function get_components(position)
            Dim oAxisComponentsArray(11)
            position.GetComponents oAxisComponentsArray
            get_components = oAxisComponentsArray
        End Function
        """

        system_service = self.application.system_service
        return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_components(self, i_axis_components_array: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetComponents(CATSafeArrayVariant
                | iAxisComponentsArray)
                | 
                |     Sets the components of an object's position. This sets the 3D-axis system
                |     associated with the object.
                | 
                |     Parameters:
                | 
                |         iAxisComponentsArray
                |             The array initialized with the components to set to the object's
                |             position. The first nine represent succcessively the components of the x-axis,
                |             y-axis, and z-axis. The last three represent the coordinates of the origin
                |             point. 
                | 
                |     Example:
                | 
                |           This example sets the 3D-axis system components stored
                |           in
                |          iAxisComponentsArray to
                |          the Position object associated with MyObject:
                |          
                | 
                |          Dim iAxisComponentsArray( 11 )
                |          ' x axis components
                |          iAxisComponentsArray( 0 )  = 1.000
                |          iAxisComponentsArray( 1 )  = 0
                |          iAxisComponentsArray( 2 )  = 0.707
                |          ' y axis components
                |          iAxisComponentsArray( 3 )  = 0
                |          iAxisComponentsArray( 4 )  = 0
                |          iAxisComponentsArray( 5 )  = 0.707
                |          ' z axis components
                |          iAxisComponentsArray( 6 )  = 0
                |          iAxisComponentsArray( 7 )  = -0.707
                |          iAxisComponentsArray( 8 )  = 0.707
                |          ' origin point coordinates
                |          iAxisComponentsArray( 9 )  = 1.000
                |          iAxisComponentsArray( 10 ) = 2.000
                |          iAxisComponentsArray( 11 ) = 3.000
                |          MyObject.Position.SetComponents iAxisComponentsArray

        :param tuple i_axis_components_array:
        :return: None
        :rtype: None
        """
        return self.position.SetComponents(i_axis_components_array)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_components'
        # # vba_code = """
        # # Public Function set_components(position)
        # #     Dim iAxisComponentsArray (2)
        # #     position.SetComponents iAxisComponentsArray
        # #     set_components = iAxisComponentsArray
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'Position(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.length import Length
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape


class HybridShapeRollingOffset(HybridShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         HybridShapeRollingOffset
                | 
                | The RollingOffset feature
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_rolling_offset = com_object

    @property
    def offset(self) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Offset() As Length (Read Only)
                | 
                |     Role: To get_Offset on the object.
                | 
                |     Parameters:
                | 
                |         oOffset
                |             offset value return value for CATScript applications, with
                |             (IDLRETVAL) function type 
                | 
                |     See also:
                |         Length 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :return: Length
        :rtype: Length
        """

        return Length(self.hybrid_shape_rolling_offset.Offset)

    @property
    def support(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Support() As Reference
                | 
                |     Role: To manage the support on the object.
                | 
                |     Parameters:
                | 
                |         iSupport
                |             return value for CATScript applications, with (IDLRETVAL) function
                |             type 
                | 
                |     See also:
                |         Reference 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_rolling_offset.Support)

    @support.setter
    def support(self, reference_support: Reference):
        """
        :param Reference reference_support:
        """

        self.hybrid_shape_rolling_offset.Support = reference_support.com_object

    def get_curve(self, i_pos: int) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func getCurve(long iPos) As Reference
                | 
                |     Role: To get_Curve on the object.
                | 
                |     Parameters:
                | 
                |         iPos
                |             Position 
                | 
                |     See also:
                |         long 
                |     oCurve
                |         return value for CATScript applications, with (IDLRETVAL) function type
                |         
                |     See also:
                |         Reference 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :param int i_pos:
        :return: Reference
        :rtype: Reference
        """
        return Reference(self.hybrid_shape_rolling_offset.getCurve(i_pos))

    def get_nb_curve(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func getNbCurve() As long
                | 
                |     Role: To get_NbCurve on the object.
                | 
                |     Parameters:
                | 
                |         CurvesNb
                |             Number of curves 
                | 
                |     See also:
                |         long 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :return: int
        :rtype: int
        """
        return self.hybrid_shape_rolling_offset.getNbCurve()

    def get_offset(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func getOffset() As double
                | 
                |     Role: To getOffset on the object.
                | 
                |     Parameters:
                | 
                |         oOffset
                |             offset value 
                | 
                |     See also:
                |         double 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :return: float
        :rtype: float
        """
        return self.hybrid_shape_rolling_offset.getOffset()

    def put_curve(self, i_curve: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub putCurve(Reference iCurve)
                | 
                |     Role: To add or remove a curve on the object.
                | 
                |     Parameters:
                | 
                |         iCurve
                |             Curve to Add/Remove if not present in the list, or to remove.
                |             
                | 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :param Reference i_curve:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_rolling_offset.putCurve(i_curve.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'put_curve'
        # # vba_code = """
        # # Public Function put_curve(hybrid_shape_rolling_offset)
        # #     Dim iCurve (2)
        # #     hybrid_shape_rolling_offset.putCurve iCurve
        # #     put_curve = iCurve
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def put_offset(self, i_offset: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub putOffset(double iOffset)
                | 
                |     Role: To put_Offset on the object.
                | 
                |     Parameters:
                | 
                |         iOffset
                |             offset value 
                | 
                |     See also:
                |         double 
                |     Returns:
                |         HRESULT S_OK if Ok E_FAIL else return error code for C++
                |         Implementations 
                |     See also:
                |         HybridShapeFactory

        :param float i_offset:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_rolling_offset.putOffset(i_offset)

    def __repr__(self):
        return f'HybridShapeRollingOffset(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pycatia.drafting_interfaces.drawing_picture import DrawingPicture
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class DrawingPictures(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     DrawingPictures
                | 
                | A collection of all the drawing pictures currently managed by a drawing view of
                | drawing sheet in a drawing document.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object, child_object=DrawingPicture)
        self.drawing_pictures = com_object

    def add(self, i_drawing_picture_path: str, i_position_x: float, i_position_y: float) -> DrawingPicture:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Add(CATBSTR iDrawingPicturePath,
                | double iPositionX,
                | double iPositionY) As DrawingPicture
                | 
                |     Inserts a drawing picture in the drawing view and adds it to the
                |     DrawingPictures collection.
                | 
                |     Parameters:
                | 
                |         iDrawingPicturePath
                |             The path of the picture file (ex : "C:/tmp/ball.bmp") .
                |         iPositionX,iPositionY
                |             The drawing picture x and y coordinates, expressed in millimeters,
                |             with respect to the drawing view coordinate system
                |             
                | 
                |     Returns:
                |         The inserted drawing picture 
                | 
                | Example:
                |     The following example inserts a drawing picture from a given picture file
                |     path The MyView is the active view in the active drawing
                |     sheet
                | 
                |      Dim MySheet As DrawingSheet
                |      Set MySheet = CATIA.ActiveDocument.Sheets.ActiveSheet
                |      Dim MyView As DrawingView
                |      Set MyView = MySheet.Views.ActiveView
                |      Dim MyDrawingPicture1 As DrawingPicture
                |      Set MyDrawingPicture1 = MyView.Pictures.Add("C:/tmp/ball.bmp", 100., 50.)

        :param str i_drawing_picture_path:
        :param float i_position_x:
        :param float i_position_y:
        :return: DrawingPicture
        :rtype: DrawingPicture
        """
        return DrawingPicture(self.drawing_pictures.Add(i_drawing_picture_path, i_position_x, i_position_y))

    def item(self, i_index: cat_variant) -> DrawingPicture:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Item(CATVariant iIndex) As DrawingPicture
                | 
                |     Returns a drawing picture using its index or its name from the
                |     DrawingPictures collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the drawing picture to retrieve from the
                |             collection of drawing pictures. As a numerics, this index is the rank of the
                |             drawing picture in the collection. The index of the first drawing picture in
                |             the collection is 1, and the index of the last drawing picture is Count. As a
                |             string, it is the name you assigned to the drawing picture using the
                |             
                | 
                |         AnyObject.Name property 
                |     Returns:
                |         The retrieved drawing picture 
                | 
                | Example:
                |     This example retrieves in ThisDrawingPicture the second drawing picture,
                |     MyView in the drawing view collection of the active sheet in the active
                |     document, supposed to be a drawing document.
                | 
                |      Dim MySheet As DrawingSheet
                |      Set MySheet = CATIA.ActiveDocument.Sheets.ActiveSheet
                |      Dim MyView  As DrawingView
                |      Set MyView  = MySheet.Views.ActiveView
                |      Dim ThisDrawingPicture As DrawingPicture
                |      Set ThisDrawingPicture = MyView.Pictures.Item(2)

        :param cat_variant i_index:
        :return: DrawingPicture
        :rtype: DrawingPicture
        """
        return DrawingPicture(self.drawing_pictures.Item(i_index))

    def remove(self, i_index: cat_variant) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Remove(CATVariant iIndex)
                | 
                |     Removes a drawing picture from the DrawingPictures
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index of the drawing picture to remove from the collection of
                |             drawing pictures. As a numerics, this index is the rank of the drawing picture
                |             in the collection. The index of the first drawing picture in the collection is
                |             1, and the index of the last drawing picture is Count. As a string, it is the
                |             name you assigned to the drawing picture using the
                |             
                | 
                |         AnyObject.Name property 
                | 
                | Example:
                |     The following example removes the third drawing picture in the drawing
                |     pictures collection of the active view of the active document, supposed to be a
                |     drawing document.
                | 
                |      Dim MyView As DrawingView
                |      Set MyView  = MySheet.Views.ActiveView
                |      MyView.Pictures.Remove(3)

        :param cat_variant i_index:
        :return: None
        :rtype: None
        """
        return self.drawing_pictures.Remove(i_index)

    def __getitem__(self, n: int) -> DrawingPicture:
        if (n + 1) > self.count:
            raise StopIteration

        return DrawingPicture(self.drawing_pictures.item(n + 1))

    def __iter__(self) -> Iterator[DrawingPicture]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'DrawingPictures(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pycatia.part_interfaces.defeaturing_filter import DefeaturingFilter
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class DefeaturingFilters(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     DefeaturingFilters
                | 
                | Represents the filter collection of a defeaturing object.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.defeaturing_filters = com_object

    def add(self, i_filter_type_to_add: str) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func Add(CATBSTR iFilterTypeToAdd) As long
                | 
                |     Creates a new filter and adds it to the Defeaturing filters
                |     collection.
                | 
                |     Parameters:
                | 
                |         iFilterTypeToAdd
                |             The type of the new filter to add among : - "DefeaturingFilletFilter" -
                |             "DefeaturingHoleFilter" - or any user-defined filter's type
                | 
                |     Returns:
                |         oAddedFilterIndex The added filter's index - equals to 0 if
                |         FAILED
                | 
                |         Example:
                |             The following example adds a new filter of type theFilterType to
                |             defeaturing colelction firstDefeaturingFilters and returns the index theIndex
                |             of the new filter
                | 
                |              Set theIndex = firstDefeaturingFilters.Add(theFilterType)

        :param str i_filter_type_to_add:
        :return: int
        :rtype: int
        """
        return self.defeaturing_filters.Add(i_filter_type_to_add)

    def item(self, i_filter_id: cat_variant) -> DefeaturingFilter:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func Item(CATVariant iFilterId) As DefeaturingFilter
                | 
                |     Returns the filter of the Defeaturing filters collection using its index or
                |     its name.
                | 
                |     Parameters:
                | 
                |         iFilterId
                |             The index or the name of the filter to retrieve As a numerics, must
                |             be in [1;Count]) 
                | 
                |     Returns:
                |         oFilter The filter (see DefeaturingFilter for list of possible
                |         actions)
                | 
                |         Example:
                |             The following example returns in myFilter the filter number
                |             theIndex of Defeaturing collection
                |             firstDefeaturingFilters:
                | 
                |              Set myFilter = firstDefeaturingFilters.Item(theIndex)

        :param cat_variant i_filter_id:
        :return: DefeaturingFilter
        :rtype: DefeaturingFilter
        """
        return DefeaturingFilter(self.defeaturing_filters.Item(i_filter_id))

    def remove(self, i_filter_id: cat_variant) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub Remove(CATVariant iFilterId)
                | 
                |     Removes a filter from the Defeaturing filters collection and deletes it,
                |     using its index or its name.
                | 
                |     Parameters:
                | 
                |         iFilterId
                |             The index or the name of the filter to retrieve As a numerics, must
                |             be in [1;Count])
                | 
                |             Example:
                |                 The two following examples remove the filter number theIndex
                |                 from Defeaturing collection
                |                 firstDefeaturingFilters:
                | 
                |                  Call firstDefeaturingFilters.Remove(theIndex)
                |                  firstDefeaturingFilters.Remove theIndex

        :param cat_variant i_filter_id:
        :return: None
        :rtype: None
        """
        return self.defeaturing_filters.Remove(i_filter_id)

    def __getitem__(self, n: int) -> DefeaturingFilter:
        if (n + 1) > self.count:
            raise StopIteration

        return DefeaturingFilter(self.defeaturing_filters.item(n + 1))

    def __iter__(self) -> Iterator[DefeaturingFilter]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'DefeaturingFilters(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.

"""

from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.angle import Angle
from pycatia.system_interfaces.any_object import AnyObject


class AxisSystem(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     AxisSystem
                |
                | The object Axis System A axis system has got one origin point and three
                | orthogonal axes, crossing at the origin point.

    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.axis_system = com_object

    @property
    def axis_rotation_angle(self) -> Angle:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property AxisRotationAngle() As Angle (Read Only)
                | 
                |     Returns the rotation angle of an axis system. Succeeds only if the axis
                |     system is defined by a rotation around an axis, wich means that its type is
                |     catAxisSystemAxisRotation.

        :return: Angle
        :rtype: Angle
        """

        return Angle(self.axis_system.AxisRotationAngle)

    @property
    def axis_rotation_reference(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property AxisRotationReference() As Reference
                | 
                |     Returns the reference for the axis rotation. Succeeds only if the axis
                |     system is defined by a rotation around an axis, wich means that its type is
                |     catAxisSystemAxisRotation.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.axis_system.AxisRotationReference)

    @axis_rotation_reference.setter
    def axis_rotation_reference(self, value: Reference):
        """
        :param Reference value:
        """

        self.axis_system.AxisRotationReference = value

    @property
    def is_current(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property IsCurrent() As boolean
                | 
                |     Returns True if the axis system is the current one, else returns False.
                |     Sets the axis system as the current one or not.
                |
                |     Example:
                |         The following example returns in isCurrent True if the axis system
                |         axisSystem is the current one :
                |
                |          isCurrent = axisSystem.IsCurrent
                |
                |         The following example sets the axis system axisSystem as the current
                |         one :
                |
                |          axisSystem.IsCurrent = 1
                |
                |         The following example sets the axis system axisSystem as not the
                |         current one :
                |
                |          axisSystem.IsCurrent = 0

        :return: bool
        :rtype: bool
        """

        return self.axis_system.IsCurrent

    @is_current.setter
    def is_current(self, value: bool):
        """
        :param bool value:
        """

        self.axis_system.IsCurrent = value

    @property
    def origin_point(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property OriginPoint() As Reference
                | 
                |     Returns or sets the geometric point which defines the origin of the axis
                |     system.
                |     OriginPoint is and must be a reference on a geometric 3D
                |     point.
                |
                |     Example:
                |         The following example sets the point Point.1 of the Geometrical Set
                |         Geometrical Set.1 as the origin point of the axis system
                |         AxisSystem0:
                |
                |      Dim HybridBody4 As AnyObject
                |      Set HybridBody4 = Body1.HybridBodies.Item  ( "Geometrical Set.1" )
                |      Dim HybridShapePointCoord5 As AnyObject
                |      Set HybridShapePointCoord5 = HybridBody4.HybridShapes.Item  ( "Point.1" )
                |      Dim Reference6 As Reference
                |      Set Reference6 = CATIA.ActiveDocument.Part.CreateReferenceFromGeometry(HybridShapePointCoord5)
                |      AxisSystem0.OriginPoint = Reference6

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.axis_system.OriginPoint)

    @origin_point.setter
    def origin_point(self, value: Reference):
        """
        :param Reference value:
        """

        self.axis_system.OriginPoint = value

    @property
    def origin_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property OriginType() As CATAxisSystemOriginType
                | 
                |     Returns or sets the way the origin point is defined.
                |     The origin point can be not specified, or be defined by coordinates or by a
                |     geometric point.
                |     CATAxisSystemOriginType is the enumeration which describes how the origin
                |     point is defined :
                |     If OriginType=0, the origin point is defined by a geometric point. If no
                |     point si selected, the origin will be automatically put at the intersection of
                |     the lines or planes defining the axes.
                |     If OriginType=1, the origin is defined by three coordinates x,y,z. Then,
                |     the origin will allways stays at the position defined by the
                |     coordinates.
                | 
                |     Example:
                |         The following example prints the origin type :
                | 
                |          Catia.SystemService.Print " OriginType = " & axisSystem.OriginType
                | 
                |         The following example sets the origin type to 1 :
                | 
                |          axisSystem.OriginType = 1

        :return: int
        :rtype: int
        """

        return self.axis_system.OriginType

    @origin_type.setter
    def origin_type(self, value: int):
        """
        :param int value:
        """

        self.axis_system.OriginType = value

    @property
    def type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Type() As CATAxisSystemMainType
                | 
                |     Returns or sets the type of the axis system. Sets the axis system
                |     type.
                | 
                |     Example:
                |         The following example returns in type1 the type of the axis system
                |         axisSystem1:
                | 
                |          type1 = axisSystem1.Type
                | 
                |         The following example sets the type of the axis system axisSystem1 as
                |         standard:
                | 
                |          axisSystem1.Type = 0
                | 
                |         The following example sets the type of the axis system axisSystem1 as
                |         axis rotation:
                | 
                |          axisSystem1.Type = 1
                | 
                |     The following example sets the type of the axis system axisSystem1 as datum
                |     (explicit):
                | 
                |      axisSystem1.Type = 3

        :return: int
        :rtype: int
        """

        return self.axis_system.Type

    @type.setter
    def type(self, value: int):
        """
        :param int value:
        """

        self.axis_system.Type = value

    @property
    def x_axis_direction(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property XAxisDirection() As Reference
                | 
                |     Reads or sets the geometric point, line or plane which defines the
                |     direction of the X axis.
                |     AxisDirection is and must be a reference on a 3D point or 3D line or
                |     plane.
                | 
                |     Example:
                |         The following example sets the point Point.1 of the Geometrical Set
                |         Geometrical Set.1 as the direction of the X axis of the axis system
                |         AxisSystem0:
                | 
                |      Dim HybridBody4 As AnyObject
                |      Set HybridBody4 = Body1.HybridBodies.Item  ( "Geometrical Set.1" )
                |      Dim HybridShapePointCoord5 As AnyObject
                |      Set HybridShapePointCoord5 = HybridBody4.HybridShapes.Item  ( "Point.1" )
                |      Dim Reference6 As Reference
                |      Set Reference6 = CATIA.ActiveDocument.Part.CreateReferenceFromGeometry( HybridShapePointCoord5 )
                |      AxisSystem0.XAxisDirection = Reference6

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.axis_system.XAxisDirection)

    @x_axis_direction.setter
    def x_axis_direction(self, value: Reference):
        """
        :param Reference value:
        """

        self.axis_system.XAxisDirection = value

    @property
    def x_axis_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property XAxisType() As CATAxisSystemAxisType
                | 
                |     Reads or sets the way the X axis is specified.
                |     An axis X,Y, or Z of the axis system can be defined by a geometric point,
                |     line or plane, or by coordinates.
                |     AxisType = 0 : The axis is defined by a geometric point, line or plane and with the same
                |                    direction.
                |     AxisType = 1 : The axis direction is defined by the three coordinates x,y,z, of a vector, to
                |                    which the axis will always stay parallel.
                |     AxisType = 2 : the axis is defined by a geometric point, line or plane and with the opposite
                |                    direction. Notice : If the X axis is neither defined by coordinates nor by a
                |                    point,line or plane, the axis will be automatically computed in order to build an
                |                    orthogonal axis system with the other specified axes.
                | 
                |     Example:
                |         The following example prints the X axis type :
                | 
                |          Catia.SystemService.Print " XAxisType = " & axisSystem.XAxisType
                | 
                |         The following example sets the X axis type to 1 :
                | 
                |          axisSystem.XAxisType = 1

        :return: int
        :rtype: int
        """

        return self.axis_system.XAxisType

    @x_axis_type.setter
    def x_axis_type(self, value: int):
        """
        :param int value:
        """

        self.axis_system.XAxisType = value

    @property
    def y_axis_direction(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property YAxisDirection() As Reference
                | 
                |     Reads or sets the geometric point, line or plane which defines the
                |     direction of the Y axis.
                |     AxisDirection is and must be a reference on a 3D point or 3D line or
                |     plane.
                | 
                |     Example:
                |         The following example sets the point Point.1 of the Geometrical Set
                |         Geometrical Set.1 as the direction of the Y axis of the axis system
                |         AxisSystem0:
                | 
                |          Dim HybridBody4 As AnyObject
                |          Set HybridBody4 = Body1.HybridBodies.Item  ( "Geometrical Set.1" ) 
                |          Dim HybridShapePointCoord5 As AnyObject
                |          Set HybridShapePointCoord5 = HybridBody4.HybridShapes.Item  ( "Point.1" ) 
                |          Dim Reference6 As Reference
                |          Set Reference6 = CATIA.ActiveDocument.Part.
                |                               CreateReferenceFromGeometry(HybridShapePointCoord5 )
                |          AxisSystem0.YAxisDirection = Reference6

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.axis_system.YAxisDirection)

    @y_axis_direction.setter
    def y_axis_direction(self, value: Reference):
        """
        :param Reference value:
        """

        self.axis_system.YAxisDirection = value

    @property
    def y_axis_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property YAxisType() As CATAxisSystemAxisType
                | 
                |     Reads or sets the way the Y axis is specified.
                |     An axis X,Y, or Z of the axis system can be defined by a geometric point,
                |     line or plane, or by coordinates.
                |     AxisType = 0 : The axis is defined by a geometric point, line or plane and with the same
                |                    direction.
                |     AxisType = 1 : The axis direction is defined by the three coordinates x,y,z, of a vector, to
                |                    which the axis will always stay parallel.
                |     AxisType = 2 : the axis is defined by a geometric point, line or plane and with the opposite
                |                    direction. Notice : If the Y axis is neither defined by coordinates nor by a
                |                    point,line or plane, the axis will be automatically computed in order to build an
                |                    orthogonal axis system with the other specified axes.
                | 
                |     Example:
                |         The following example prints the Y axis type :
                | 
                |          Catia.SystemService.Print " YAxisType = " & axisSystem.YAxisType
                | 
                |         The following example sets the Y axis type to 1 :
                | 
                |          axisSystem.YAxisType = 1

        :return: int
        :rtype: int
        """

        return self.axis_system.YAxisType

    @y_axis_type.setter
    def y_axis_type(self, value: int):
        """
        :param int value:
        """

        self.axis_system.YAxisType = value

    @property
    def z_axis_direction(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ZAxisDirection() As Reference
                | 
                |     Reads or sets the geometric point, line or plane which defines the
                |     direction of the Z axis.
                |     AxisDirection is and must be a reference on a 3D point or 3D line or
                |     plane.
                | 
                |     Example:
                |         The following example sets the point Point.1 of the Geometrical Set
                |         Geometrical Set.1 as the direction of the Z axis of the axis system
                |         AxisSystem0:
                | 
                |          Dim HybridBody4 As AnyObject
                |          Set HybridBody4 = Body1.HybridBodies.Item  ( "Geometrical Set.1" ) 
                |          Dim HybridShapePointCoord5 As AnyObject
                |          Set HybridShapePointCoord5 = HybridBody4.HybridShapes.Item  ( "Point.1" ) 
                |          Dim Reference6 As Reference
                |          Set Reference6 = CATIA.ActiveDocument.Part.
                |                               CreateReferenceFromGeometry(HybridShapePointCoord5)
                |          AxisSystem0.ZAxisDirection = Reference6

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.axis_system.ZAxisDirection)

    @z_axis_direction.setter
    def z_axis_direction(self, value: Reference):
        """
        :param Reference value:
        """

        self.axis_system.ZAxisDirection = value

    @property
    def z_axis_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ZAxisType() As CATAxisSystemAxisType
                | 
                |     Reads or sets the way the Z axis is specified.
                |     An axis X,Y, or Z of the axis system can be defined by a geometric point,
                |     line or plane, or by coordinates.
                |     AxisType = 0 : The axis is defined by a geometric point, line or plane and with the same
                |                    direction.
                |     AxisType = 1 : The axis direction is defined by the three coordinates x,y,z, of a vector, to
                |                    which the axis will always stay parallel.
                |     AxisType = 2 : the axis is defined by a geometric point, line or plane and with the opposite
                |                    direction. Notice : If the Z axis is neither defined by coordinates nor by a
                |                    point,line or plane, the axis will be automatically computed in order to build an
                |                    orthogonal axis system with the other specified axes.
                | 
                |     Example:
                |         The following example prints the Z axis type :
                | 
                |          Catia.SystemService.Print " ZAxisType = " & axisSystem.ZAxisType
                | 
                |         The following example sets the Z axis type to 1 :
                | 
                |          axisSystem.ZAxisType = 1

        :return: int
        :rtype: int
        """

        return self.axis_system.ZAxisType

    @z_axis_type.setter
    def z_axis_type(self, value: int):
        """
        :param int value:
        """

        self.axis_system.ZAxisType = value

    def get_euler_angles(self) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetEulerAngles(Angle oFirstAngle,
                | Angle oSecondAngle,
                | Angle ThirdAngle)
                | 
                |     Returns the Euler Angles of an axis system. Succeeds only if the axis
                |     system is defined by Euler angles, wich means its type is
                |     catAxisSystemEulerAngles.

        :param Angle o_first_angle:
        :param Angle o_second_angle:
        :param Angle third_angle:
        :return: None
        :rtype: None
        """

        vba_function_name = 'get_euler_angles'
        vba_code = """
        Public Function get_euler_angles(axis_system)
            Dim oFirstAngle (2)
            axis_system.GetEulerAngles oFirstAngle
            get_euler_angles = oFirstAngle
        End Function
        """

        system_service = self.application.system_service
        return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_origin(self) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetOrigin(CATSafeArrayVariant oOrigin)
                | 
                |     Returns the coordinates X,Y,Z of the origin point of the axis
                |     system.
                | 
                |     Parameters:
                | 
                |         oOrigin
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the origin point of the axis system.
                |             
                | 
                |     Example:
                |         The following example retrieves in originCoord the coordinates of the
                |         origin point of the axisSystem axis system:
                | 
                |          Dim originCoord(2)
                |          axisSystem.GetOrigin originCoord

        :param tuple o_origin:
        :return: None
        :rtype: None
        """

        vba_function_name = 'get_origin'
        vba_code = """
        Public Function get_origin(axis_system)
            Dim oOrigin (2)
            axis_system.GetOrigin oOrigin
            get_origin = oOrigin
        End Function
        """

        system_service = self.application.system_service
        return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_vectors(self) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetVectors(CATSafeArrayVariant oVectorX,
                | CATSafeArrayVariant oVectorY)
                | 
                |     Returns the coordinates X,Y,Z of the axes X and Y of the axis
                |     system.
                | 
                |     Parameters:
                | 
                |         oVectorX
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the X axis vector of the axis system.
                |             
                |         oVectorY
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the Y axis vector of the axis system.
                |             
                | 
                |     Example:
                |         The following example retrieves in vectorXCoord and vectorYCoord the
                |         coordinates of the vectors of the axisSystem axis
                |         system:
                | 
                |          Dim vectorXCoord(2)
                |          Dim vectorYCoord(2)
                |          axisSystem.GetVectors vectorXCoord, vectorYCoord

        :param tuple o_vector_x:
        :param tuple o_vector_y:
        :return: None
        :rtype: None
        """

        vba_function_name = 'get_vectors'
        vba_code = """
        Public Function get_vectors(axis_system)
            Dim oVectorX (2)
            axis_system.GetVectors oVectorX
            get_vectors = oVectorX
        End Function
        """

        system_service = self.application.system_service
        return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_x_axis(self) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetXAxis(CATSafeArrayVariant oXAxis)
                | 
                |     Returns the coordinates X,Y,Z of the X axis of the axis
                |     system.
                | 
                |     Parameters:
                | 
                |         oXAxis
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the X axis of the axis system.
                |             
                | 
                |     Example:
                |         The following example retrieves in XAxisCoord the coordinates of the X
                |         axis of the axisSystem axis system:
                | 
                |          Dim XAxisCoord(2)
                |          axisSystem.GetXAxis XAxisCoord

        :param tuple o_x_axis:
        :return: None
        :rtype: None
        """

        vba_function_name = 'get_x_axis'
        vba_code = """
        Public Function get_x_axis(axis_system)
            Dim oXAxis (2)
            axis_system.GetXAxis oXAxis
            get_x_axis = oXAxis
        End Function
        """

        system_service = self.application.system_service
        return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_y_axis(self) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetYAxis(CATSafeArrayVariant oYAxis)
                | 
                |     Returns the coordinates X,Y,Z of the Y axis of the axis
                |     system.
                | 
                |     Parameters:
                | 
                |         oYAxis
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the Y axis of the axis system.
                |             
                | 
                |     Example:
                |         The following example retrieves in YAxisCoord the coordinates of the Y
                |         axis of the axisSystem axis system:
                | 
                |          Dim YAxisCoord(2)
                |          axisSystem.GetYAxis XAxisCoord

        :param tuple o_y_axis:
        :return: None
        :rtype: None
        """

        vba_function_name = 'get_y_axis'
        vba_code = """
        Public Function get_y_axis(axis_system)
            Dim oYAxis (2)
            axis_system.GetYAxis oYAxis
            get_y_axis = oYAxis
        End Function
        """

        system_service = self.application.system_service
        return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_z_axis(self) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetZAxis(CATSafeArrayVariant oZAxis)
                | 
                |     Returns the coordinates X,Y,Z of the Z axis of the axis
                |     system.
                | 
                |     Parameters:
                | 
                |         oZAxis
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the Z axis of the axis system.
                |             
                | 
                |     Example:
                |         The following example retrieves in ZAxisCoord the coordinates of the Z
                |         axis of the axisSystem axis system:
                | 
                |          Dim ZAxisCoord(2)
                |          axisSystem.GetZAxis ZAxisCoord

        :param tuple o_z_axis:
        :return: None
        :rtype: None
        """

        vba_function_name = 'get_z_axis'
        vba_code = """
        Public Function get_z_axis(axis_system)
            Dim oZAxis (2)
            axis_system.GetZAxis oZAxis
            get_z_axis = oZAxis
        End Function
        """

        system_service = self.application.system_service
        return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def put_origin(self, i_origin: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub PutOrigin(CATSafeArrayVariant iOrigin)
                | 
                |     Defines the coordinates X,Y,Z of the origin point of the axis
                |     system.
                | 
                |     Parameters:
                | 
                |         iOrigin
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the origin point of the axis system.
                |             
                | 
                |     Example:
                |         The following example puts in originCoord the new coordinates of the
                |         origin point of the axisSystem axis system:
                | 
                |          Dim originCoord(2)
                |          originCoord ( 0 )  = 100.000000
                |          originCoord ( 1 )  = 200.000000
                |          originCoord ( 2 )  = 10.000000
                |          axisSystem.PutOrigin originCoord

        :param tuple i_origin:
        :return: None
        :rtype: None
        """
        return self.axis_system.PutOrigin(i_origin)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'put_origin'
        # # vba_code = """
        # # Public Function put_origin(axis_system)
        # #     Dim iOrigin (2)
        # #     axis_system.PutOrigin iOrigin
        # #     put_origin = iOrigin
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def put_vectors(self, i_vector_x: tuple, i_vector_y: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub PutVectors(CATSafeArrayVariant iVectorX,
                | CATSafeArrayVariant iVectorY)
                | 
                |     Defines the coordinates X,Y,Z of the axes X and Y of the axis
                |     system.
                | 
                |     Parameters:
                | 
                |         iVectorX
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the X axis vector of the axis system.
                |             
                |         iVectorY
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the Y axis vector of the axis system.
                |             
                | 
                |     Example:
                |         The following example modifies in vectorXCoord and vectorYCoord the
                |         coordinates of the vectors of the axisSystem axis
                |         system:
                | 
                |          Dim vectorXCoord(2)
                |          vectorYCoord ( 0 )  = 1.000000
                |          vectorYCoord ( 1 )  = -1.000000
                |          vectorYCoord ( 2 )  = 0.000000
                |          Dim vectorYCoord(2)
                |          vectorYCoord ( 0 )  = 0.000000
                |          vectorYCoord ( 1 )  = 0.000000
                |          vectorYCoord ( 2 )  = 1.000000
                |          axisSystem.PutVectors vectorXCoord, vectorYCoord

        :param tuple i_vector_x:
        :param tuple i_vector_y:
        :return: None
        :rtype: None
        """
        return self.axis_system.PutVectors(i_vector_x, i_vector_y)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'put_vectors'
        # # vba_code = """
        # # Public Function put_vectors(axis_system)
        # #     Dim iVectorX (2)
        # #     axis_system.PutVectors iVectorX
        # #     put_vectors = iVectorX
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def put_x_axis(self, i_x_axis: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub PutXAxis(CATSafeArrayVariant iXAxis)
                | 
                |     Defines the coordinates X,Y,Z of the X axis of the axis
                |     system.
                | 
                |     Parameters:
                | 
                |         iXAxis
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the X axis of the axis system.
                |             
                | 
                |     Example:
                |         The following example puts in XAxisCoord the new coordinates of the X
                |         axis of the axisSystem axis system:
                | 
                |          Dim XAxis(2)
                |          XAxis ( 0 )  = 100.000000
                |          XAxis ( 1 )  = 200.000000
                |          XAxis ( 2 )  = 10.000000
                |          axisSystem.PutXAxis XAxis

        :param tuple i_x_axis:
        :return: None
        :rtype: None
        """
        return self.axis_system.PutXAxis(i_x_axis)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'put_x_axis'
        # # vba_code = """
        # # Public Function put_x_axis(axis_system)
        # #     Dim iXAxis (2)
        # #     axis_system.PutXAxis iXAxis
        # #     put_x_axis = iXAxis
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def put_y_axis(self, i_y_axis: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub PutYAxis(CATSafeArrayVariant iYAxis)
                | 
                |     Defines the coordinates X,Y,Z of the Y axis of the axis
                |     system.
                | 
                |     Parameters:
                | 
                |         iYAxis
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the Y axis of the axis system.
                |             
                | 
                |     Example:
                |         The following example puts in XAxisCoord the new coordinates of the Y
                |         axis of the axisSystem axis system:
                | 
                |          Dim YAxis(2)
                |          YAxis ( 0 )  = 100.000000
                |          YAxis ( 1 )  = 200.000000
                |          YAxis ( 2 )  = 10.000000
                |          axisSystem.PutYAxis YAxis

        :param tuple i_y_axis:
        :return: None
        :rtype: None
        """
        return self.axis_system.PutYAxis(i_y_axis)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'put_y_axis'
        # # vba_code = """
        # # Public Function put_y_axis(axis_system)
        # #     Dim iYAxis (2)
        # #     axis_system.PutYAxis iYAxis
        # #     put_y_axis = iYAxis
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def put_z_axis(self, i_z_axis: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub PutZAxis(CATSafeArrayVariant iZAxis)
                | 
                |     Defines the coordinates X,Y,Z of the Z axis of the axis
                |     system.
                | 
                |     Parameters:
                | 
                |         iZAxis
                |             A Safe Array made up of 3 doubles: X, Y, Z, representing the
                |             coordinates in model space of the Z axis of the axis system.
                |             
                | 
                |     Example:
                |         The following example puts in ZAxisCoord the new coordinates of the Z
                |         axis of the axisSystem axis system:
                | 
                |          Dim ZAxis(2)
                |          ZAxis ( 0 )  = 100.000000
                |          ZAxis ( 1 )  = 200.000000
                |          ZAxis ( 2 )  = 10.000000
                |          axisSystem.PutZAxis ZAxis

        :param tuple i_z_axis:
        :return: None
        :rtype: None
        """
        return self.axis_system.PutZAxis(i_z_axis)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'put_z_axis'
        # # vba_code = """
        # # Public Function put_z_axis(axis_system)
        # #     Dim iZAxis (2)
        # #     axis_system.PutZAxis iZAxis
        # #     put_z_axis = iZAxis
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'AxisSystem(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pycatia.in_interfaces.light_source import LightSource
from pycatia.system_interfaces.collection import Collection


class LightSources(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     LightSources
                | 
                | A collection of all the LightSource objects.
                | This collection is currently managed by a Viewer3D object.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object, child_object=LightSource)
        self.light_sources = com_object

    def add(self) -> LightSource:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Add() As LightSource
                | 
                |     Adds a new light source to the LightSources collection. 
                | 
                | Example:
                |     The following adds a light source to the collection attached to the active
                |     viewer. This viewer must be a @see Viewer3D object.
                | 
                |      Dim MyViewer As Viewer
                |      Set MyViewer = CATIA.ActiveWindow.ActiveViewer
                |      Dim MyLightSource As LightSource
                |      Set MyLightSource = MyViewer.LightSources.Add

        :return: LightSource
        :rtype: LightSource
        """
        return LightSource(self.light_sources.Add())

    def item(self, i_index: int) -> LightSource:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func Item(long iIndex) As LightSource
                | 
                |     Returns a light source from its index in the LightSources
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index of the light source to retrieve in the collection of
                |             light sources. Compared with other collections, you cannot use the name of the
                |             light source as argument. 
                | 
                |     Returns:
                |         The retrieved light source 
                | 
                | Example:
                |     The following example returns in MyLightSource the sixth light source in
                |     the collection.
                | 
                |      Dim MyLightSource As LightSource
                |      Set MyLightSource = LightSources.Item(6)

        :param int i_index:
        :return: LightSource
        :rtype: LightSource
        """
        return LightSource(self.light_sources.Item(i_index))

    def remove(self, i_index: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub Remove(long iIndex)
                | 
                |     Removes a light source from the LightSources collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index of the light source to remove. Compared with other
                |             collections, you cannot use the name of the light source as argument.
                |             
                | 
                |     Example:
                |         The following example removes the second light source in the collection
                |         attached to the active viewer. This viewer must be a 
                | 
                |     Viewer3D object.
                | 
                |      Dim MyViewer As Viewer
                |      Set MyViewer = CATIA.ActiveWindow.ActiveViewer
                |      MyViewer.LightSources.Remove(2)

        :param int i_index:
        :return: None
        :rtype: None
        """
        return self.light_sources.Remove(i_index)

    def __getitem__(self, n: int) -> LightSource:
        if (n + 1) > self.count:
            raise StopIteration

        return LightSource(self.light_sources.item(n + 1))

    def __iter__(self) -> Iterator[LightSource]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'LightSources(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.reference import Reference
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape


class HybridShapeWrapSurface(HybridShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         HybridShapeWrapSurface
                | 
                | Represents the hybrid shape wrap surface object.
                | Role: To access the data of the hybrid shape wrap surface
                | object.
                | 
                | This data includes:
                | 
                |     Two definition surfaces (refrence and target), who define the
                |     deformation
                | 
                | Use the CATIAHybridShapeFactory to create a HybridShapeWrapSurface
                | object.
                | 
                | See also:
                |     HybridShapeFactory
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_wrap_surface = com_object

    @property
    def deformation_mode(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property DeformationMode() As long
                | 
                |     Returns or sets whether the wrap surface is or should be created as
                |     a"Normal" or with a "3D" deformation mode.
                |     Legal values: 2 for the normal solution and 1 for 3D
                |     solution.
                | 
                |     Example:
                | 
                |           This example sets the mode to create the wrap
                |           surface
                |          hybWrapSurface with a 3D deformation mode.
                |          
                | 
                |          hybWrapSurface.3D deformation mode = 1

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_wrap_surface.DeformationMode

    @deformation_mode.setter
    def deformation_mode(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_wrap_surface.DeformationMode = value

    @property
    def reference_surface(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ReferenceSurface() As Reference
                | 
                |     Returns or sets the reference surface of the WrapSurface. 
                | 
                | Example:
                |     This example retrieves in ReferenceSurface the surface to deform of the
                |     ShpWrapSurface hybrid shape WrapSurface feature.
                | 
                |      ReferenceSurface = ShpWrapSurface.Surface

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_wrap_surface.ReferenceSurface)

    @reference_surface.setter
    def reference_surface(self, reference_surface: Reference):
        """
        :param Reference reference_surface:
        """

        self.hybrid_shape_wrap_surface.ReferenceSurface = reference_surface.com_object

    @property
    def surface(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Surface() As Reference
                | 
                |     Returns or sets the reference surface to deform of the WrapSurface.
                |     
                | 
                | Example:
                |     This example retrieves in SurfaceToDeform the surface to deform of the
                |     ShpWrapSurface hybrid shape WrapSurface feature.
                | 
                |      SurfaceToDeform = ShpWrapSurface.Surface

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_wrap_surface.Surface)

    @surface.setter
    def surface(self, reference_surface: Reference):
        """
        :param Reference reference_surface:
        """

        self.hybrid_shape_wrap_surface.Surface = reference_surface.com_object

    @property
    def target_surface(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property TargetSurface() As Reference
                | 
                |     Returns or sets the target surface of the WrapSurface. 
                | 
                | Example:
                |     This example retrieves in TargetSurface the surface to deform of the
                |     ShpWrapSurface hybrid shape WrapSurface feature.
                | 
                |      TargetSurface = ShpWrapSurface.Surface

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_wrap_surface.TargetSurface)

    @target_surface.setter
    def target_surface(self, reference_surface: Reference):
        """
        :param Reference reference_surface:
        """

        self.hybrid_shape_wrap_surface.TargetSurface = reference_surface.com_object

    def __repr__(self):
        return f'HybridShapeWrapSurface(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.system_interfaces.any_object import AnyObject
from pycatia.types.general import cat_variant


class AnalysisMaterial(AnyObject):
    """
    .. note::
        :class: toggle

        CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

            | System.IUnknown
            |     System.IDispatch
            |         System.CATBaseUnknown
            |             System.CATBaseDispatch
            |                 System.AnyObject
            |                     AnalysisMaterial
            |
            | Represents an AnalysisMaterial object.
            | This object is used to manage the Analysis properties of a
            | material.

    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.analysis_material = com_object

    def get_type(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetType() As CATBSTR
                |
                |     Returns the analysis type (Ex: MATERIAL_ORTHOTROPIC, MATERIAL_ISOTROPIC).

        :return: str
        :rtype: str
        """
        return self.analysis_material.GetType()

    def get_value(self, i_label: str) -> cat_variant:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetValue(CATBSTR iLabel) As CATVariant
                |
                |     Returns an analysis value of the material. The name of the parameter is
                |     needed

        :param str i_label:
        :return: cat_variant
        :rtype: cat_variant
        """
        return self.analysis_material.GetValue(i_label)

    def put_value(self, i_label: str, i_value: cat_variant) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub PutValue(CATBSTR iLabel,
                | CATVariant iValue)
                |
                |     Sets an analysis value of the material. The name of the
                |     parameter is needed

        :param str i_label:
        :param cat_variant i_value:
        :return: None
        :rtype: None
        """
        return self.analysis_material.PutValue(i_label, i_value)

    def __repr__(self):
        return f'AnalysisMaterial(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pywintypes import com_error

from pycatia.exception_handling.exceptions import CATIAApplicationException
from pycatia.mec_mod_interfaces.hybrid_body import HybridBody
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class HybridBodies(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     HybridBodies
                | 
                | A collection of the HybridBody objects.
    
    """

    def __init__(self, com_object, child_object=HybridBody):
        super().__init__(com_object, child_object=HybridBody)
        self.hybrid_bodies = com_object
        self.child_object = child_object

    def add(self) -> HybridBody:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Add() As HybridBody
                | 
                |     Creates a new hybrid body and adds it to the HybridBodies collection. This
                |     body becomes the current one
                | 
                |     Returns:
                |         The created body 
                |     Example:
                |         The following example creates a body named newHybridBody in the hybrid
                |         body collection of the rootPart part in the partDoc part document. NewPartBody
                |         becomes the current body in partDoc.
                | 
                |          Set NewPartBody = rootPart.Bodies.AddPartBody()

        :return: HybridBody
        :rtype: HybridBody
        """
        return HybridBody(self.hybrid_bodies.Add())

    def item(self, i_index: cat_variant) -> HybridBody:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func Item(CATVariant iIndex) As HybridBody
                | 
                |     Returns a body using its index or its name from the Bodies
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the hybrid body to retrieve from the
                |             collection of hybrid bodies. As a numerics, this index is the rank of the
                |             hybrid body in the collection. The index of the first hybrid body in the
                |             collection is 1, and the index of the last hybrid body is Count. As a string,
                |             it is the name you assigned to the hybrid body using the
                |             
                | 
                |         AnyObject.Name property. 
                |     Returns:
                |         The retrieved hybrid body 
                |     Example:
                |         This example retrieves in ThisHybridBody the fifth hybrid body in the
                |         collection and in ThatHybridBody the hybrid body named MyHybridBody in the
                |         hybrid body collection of the partDoc part document.
                | 
                |          Set hybridBodyColl = partDoc.Part.HybridBodies
                |          Set ThisHybridBody = hybridBodyColl.Item(5)
                |          Set ThatHybridBody = hybridBodyColl.Item("MyHybridBody")

        :param cat_variant i_index:
        :return: HybridBody
        :rtype: HybridBody
        """
        try:
            return HybridBody(self.hybrid_bodies.Item(i_index))
        except com_error:
            raise CATIAApplicationException(f'Could not find hybrid_body "i_index"')

    def __getitem__(self, n: int) -> HybridBody:
        if (n + 1) > self.count:
            raise StopIteration

        return HybridBody(self.hybrid_bodies.item(n + 1))

    def __iter__(self) -> Iterator[HybridBody]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'HybridBodies(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-09 09:53:18.676780

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""


class IUnknown:
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-09 09:53:18.676780)

                | Role: All interfaces derive from IUnknown which replaces for UNIX the native
                | Microsoft(R) IUnknown interface. This interface supplies the three basic
                | methods QueryInterface, AddRef and Release to be COM (Microsoft(R) Component
                | Object Model) compliant. These methods cannot be used in a
                | macro.
                | 
                | Copyright © 1999-2011, Dassault Systèmes. All rights reserved.
    
    """

    def __init__(self):
        pass

    def __repr__(self):
        return f'IUnknown()'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.in_interfaces.reference import Reference
from pycatia.in_interfaces.references import References
from pycatia.knowledge_interfaces.angle import Angle
from pycatia.knowledge_interfaces.length import Length
from pycatia.part_interfaces.dress_up_shape import DressUpShape


class Chamfer(DressUpShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.Shape
                |                         PartInterfaces.DressUpShape
                |                             Chamfer
                | 
                | Represents the chamfer shape.
                | A chamfer is made up of a list of geometrical elements to process, such as
                | faces, and is defined using a couple of parameters, such as two lengths, or a
                | length and an angle.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.chamfer = com_object

    @property
    def angle(self) -> Angle:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Angle() As Angle (Read Only)
                | 
                |     Returns the chamfer angle. This is valid only if the chamfer is defined
                |     using a length and an angle, that is if the chamfer definition mode
                |     CatChamferMode is set to catLengthAngleChamfer.
                | 
                |     Example:
                |         The following example returns in angle the angle of the firstChamfer
                |         chamfer:
                | 
                |          Set angle = firstChamfer.Angle

        :return: Angle
        :rtype: Angle
        """

        return Angle(self.chamfer.Angle)

    @property
    def elements_to_chamfer(self) -> References:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ElementsToChamfer() As References (Read Only)
                | 
                |     Returns the collection of geometrical elements to be
                |     chamfered.
                | 
                |     Example:
                |         The following example returns in list the list of elements of the
                |         firstChamfer chamfer:
                | 
                |          Set list = firstChamfer.ElementsToChamfer

        :return: References
        :rtype: References
        """

        return References(self.chamfer.ElementsToChamfer)

    @property
    def length1(self) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Length1() As Length (Read Only)
                | 
                |     Returns the chamfer first length. This is the first length if the chamfer
                |     is defined by two lengthes, or the chamfer if the chamfer is defined by a
                |     length and an angle.
                | 
                |     Example:
                |         The following example returns in length1 the first length of the
                |         firstChamfer chamfer:
                | 
                |          Set length1 = firstChamfer.Length1

        :return: Length
        :rtype: Length
        """

        return Length(self.chamfer.Length1)

    @property
    def length2(self) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Length2() As Length (Read Only)
                | 
                |     Returns the chamfer second length. This is valid only if the chamfer is
                |     defined using two lengthes, that is if the chamfer definition mode
                |     CatChamferMode is set to catTwoLengthChamfer.
                | 
                |     Example:
                |         The following example returns in length2 the second length of the
                |         firstChamfer chamfer:
                | 
                |          Set length2 = firstChamfer.Length2

        :return: Length
        :rtype: Length
        """

        return Length(self.chamfer.Length2)

    @property
    def mode(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Mode() As CatChamferMode
                | 
                |     Returns or sets the chamfer definition mode. The chamfer definition mode
                |     enables the chamfer to be defined using either two lengthes or a length and an
                |     angle.
                | 
                |     Example:
                |         The following example returns in mode how the parameters of the
                |         firstChamfer chamfer are defined, and then sets it to
                |         catTwoLengthChamfer:
                | 
                |          Set mode = firstChamfer.Mode
                |          firstChamfer.Mode = catTwoLengthChamfer

        :return: int
        :rtype: int
        """

        return self.chamfer.Mode

    @mode.setter
    def mode(self, value: int):
        """
        :param int value:
        """

        self.chamfer.Mode = value

    @property
    def orientation(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Orientation() As CatChamferOrientation
                | 
                |     Returns or sets the chamfer orientation.
                | 
                |     Example:
                |         The following example returns in orient the orientation mode of the
                |         firstChamfer chamfer, and then sets it to
                |         catReverseChamfer:
                | 
                |          Set orient = firstChamfer.Orientation
                |          firstChamfer.Orientation = catReverseChamfer

        :return: int
        :rtype: int
        """

        return self.chamfer.Orientation

    @orientation.setter
    def orientation(self, value: int):
        """
        :param int value:
        """

        self.chamfer.Orientation = value

    @property
    def propagation(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Propagation() As CatChamferPropagation
                | 
                |     Returns or sets the propagation mode of the geometrical elements to be
                |     chamfered.
                | 
                |     Example:
                |         The following example returns in prop the propagation mode of the
                |         firstChamfer chamfer, and then sets it to
                |         catMinimalChamfer:
                | 
                |          Set prop = firstChamfer.Propagation
                |          firstChamfer.Propagation = catMinimalChamfer

        :return: int
        :rtype: int
        """

        return self.chamfer.Propagation

    @propagation.setter
    def propagation(self, value: int):
        """
        :param int value:
        """

        self.chamfer.Propagation = value

    def add_element_to_chamfer(self, i_element_to_chamfer: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub AddElementToChamfer(Reference iElementToChamfer)
                | 
                |     Adds a new geometrical element to be chamfered.
                | 
                |     Parameters:
                | 
                |         iElementToChamfer
                |             The new element to be chamfered
                |             The following 
                | 
                |         Boundary object is supported: TriDimFeatEdge. 
                | 
                | Example:
                |     The following example adds the new element element to be chamfered for the
                |     firstChamfer chamfer:
                | 
                |      firstChamfer.AddElementToChamfer(element)

        :param Reference i_element_to_chamfer:
        :return: None
        :rtype: None
        """
        return self.chamfer.AddElementToChamfer(i_element_to_chamfer.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'add_element_to_chamfer'
        # # vba_code = """
        # # Public Function add_element_to_chamfer(chamfer)
        # #     Dim iElementToChamfer (2)
        # #     chamfer.AddElementToChamfer iElementToChamfer
        # #     add_element_to_chamfer = iElementToChamfer
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def withdraw_element_to_chamfer(self, i_element_to_withdraw: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub WithdrawElementToChamfer(Reference iElementToWithdraw)
                | 
                |     Withdraws a geometrical element from those to be
                |     chamfered.
                | 
                |     Parameters:
                | 
                |         iElementToWithdraw
                |             The existing element to withdraw
                |             The following 
                | 
                |         Boundary object is supported: TriDimFeatEdge. 
                | 
                | Example:
                |     The following example withdraws an existing element element to be chamfered
                |     from the firstChamfer chamfer:
                | 
                |      firstChamfer.WithdrawElementToChamfer(element)

        :param Reference i_element_to_withdraw:
        :return: None
        :rtype: None
        """
        return self.chamfer.WithdrawElementToChamfer(i_element_to_withdraw.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'withdraw_element_to_chamfer'
        # # vba_code = """
        # # Public Function withdraw_element_to_chamfer(chamfer)
        # #     Dim iElementToWithdraw (2)
        # #     chamfer.WithdrawElementToChamfer iElementToWithdraw
        # #     withdraw_element_to_chamfer = iElementToWithdraw
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'Chamfer(name="{self.name}")'
